import { EVMAccountAddress, EVMContractAddress, BlockchainCommonErrors, ERC20ContractError, TokenAmount } from "@snickerdoodlelabs/objects";
import { ethers, EventFilter } from "ethers";
import { ResultAsync } from "neverthrow";
import { BaseContract } from "../implementations/BaseContract.js";
import { IEthersContractError } from "../implementations/BlockchainErrorMapper.js";
import { ContractOverrides, IERC20Contract, WrappedTransactionResponse } from "../interfaces/index.js";
export declare class ERC20Contract extends BaseContract<ERC20ContractError> implements IERC20Contract {
    protected providerOrSigner: ethers.providers.Provider | ethers.providers.JsonRpcSigner | ethers.Wallet;
    protected contractAddress: EVMContractAddress;
    constructor(providerOrSigner: ethers.providers.Provider | ethers.providers.JsonRpcSigner | ethers.Wallet, contractAddress: EVMContractAddress);
    name(): ResultAsync<string, ERC20ContractError | BlockchainCommonErrors>;
    symbol(): ResultAsync<string, ERC20ContractError | BlockchainCommonErrors>;
    decimals(): ResultAsync<number, ERC20ContractError | BlockchainCommonErrors>;
    totalSupply(): ResultAsync<bigint, ERC20ContractError | BlockchainCommonErrors>;
    balanceOf(address: EVMAccountAddress | EVMContractAddress): ResultAsync<bigint, ERC20ContractError | BlockchainCommonErrors>;
    allowance(owner: EVMAccountAddress | EVMContractAddress, spender: EVMAccountAddress | EVMContractAddress): ResultAsync<bigint, ERC20ContractError | BlockchainCommonErrors>;
    approve(spender: EVMAccountAddress | EVMContractAddress, amount: TokenAmount, overrides?: ContractOverrides): ResultAsync<WrappedTransactionResponse, BlockchainCommonErrors | ERC20ContractError>;
    transfer(recipient: EVMAccountAddress | EVMContractAddress, amount: TokenAmount, overrides?: ContractOverrides): ResultAsync<WrappedTransactionResponse, BlockchainCommonErrors | ERC20ContractError>;
    transferFrom(sender: EVMAccountAddress | EVMContractAddress, recipient: EVMAccountAddress | EVMContractAddress, amount: TokenAmount, overrides?: ContractOverrides): ResultAsync<WrappedTransactionResponse, BlockchainCommonErrors | ERC20ContractError>;
    protected generateContractSpecificError(msg: string, e: IEthersContractError, transaction: ethers.Transaction | null): ERC20ContractError;
    filters: {
        Transfer: (fromAddress: EVMAccountAddress | null, toAddress: EVMAccountAddress | null) => EventFilter;
    };
}
//# sourceMappingURL=ERC20Contract.d.ts.map