var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var RewardsContractFactory_1;
import { EVMContractAddress, EVMAccountAddress, RewardsFactoryError, ECreatedRewardType, } from "@snickerdoodlelabs/objects";
import { ethers } from "ethers";
import { injectable } from "inversify";
import { ResultAsync } from "neverthrow";
import { BaseContract } from "../../implementations/BaseContract.js";
import { GasUtils } from "../../implementations/GasUtils.js";
import { ContractsAbis, } from "../../interfaces/objects/index.js";
let RewardsContractFactory = RewardsContractFactory_1 = class RewardsContractFactory extends BaseContract {
    constructor(providerOrSigner, rewardType) {
        super(providerOrSigner, EVMContractAddress(ethers.constants.AddressZero), // The rewards contract factory deploys a new contract, hence doesn't have a contract address
        ContractsAbis.ERC721Reward.abi);
        this.providerOrSigner = providerOrSigner;
        this.rewardType = rewardType;
        // Set the correct contract factory based on rewardTypeToDeploy
        this.contractFactory = new ethers.ContractFactory(ContractsAbis.ERC721Reward.abi, ContractsAbis.ERC721Reward.bytecode, providerOrSigner);
        this.rewardTypeToDeploy = rewardType;
    }
    // function to deploy a new ERC721 reward contract
    deployERC721Reward(name, symbol, baseURI, overrides, omitGasFee = false) {
        return GasUtils.getGasFee(this.providerOrSigner).andThen((gasFee) => {
            let contractOverrides = {
                ...gasFee,
                ...overrides,
            };
            // If the chain does not support EIP-1559, remove the gas fee override and only maintain the override passed in from the chain service
            if (omitGasFee == true) {
                contractOverrides = {
                    ...overrides,
                };
            }
            return this.writeToContractFactory("deploy", [name, symbol, baseURI], contractOverrides, true);
        });
    }
    estimateGasToDeployERC721Contract(name, symbol, baseURI) {
        return ResultAsync.fromPromise(this.providerOrSigner.estimateGas(this.contractFactory.getDeployTransaction(name, symbol, baseURI)), (e) => {
            return this.generateError(e, "Failed to wait() for contract deployment");
        }).map((estimatedGas) => {
            // Increase estimated gas buffer by 20%
            return estimatedGas.mul(120).div(100);
        });
    }
    generateContractSpecificError(msg, e, transaction) {
        return new RewardsFactoryError(msg, e, transaction);
    }
    // Takes the factory's deploy function name and params, submits the transaction and returns a WrappedTransactionResponse
    writeToContractFactory(functionName, functionParams, overrides, isDeployingContract) {
        return ResultAsync.fromPromise(this.contractFactory[functionName](...functionParams, {
            ...overrides,
        }), (e) => {
            return this.generateError(e, `Unable to call ${functionName}()`);
        }).map((transactionResponse) => {
            var _a;
            // If we are deploying a contract, the deploy() call returns an ethers.Contract object and the txresponse is under the deployTransaction property
            return RewardsContractFactory_1.buildWrappedTransactionResponse(isDeployingContract == true
                ? transactionResponse.deployTransaction
                : transactionResponse, EVMContractAddress(""), EVMAccountAddress((_a = this.providerOrSigner) === null || _a === void 0 ? void 0 : _a.address), functionName, functionParams, ContractsAbis.ERC721Reward.abi);
        });
    }
};
RewardsContractFactory = RewardsContractFactory_1 = __decorate([
    injectable(),
    __metadata("design:paramtypes", [Object, String])
], RewardsContractFactory);
export { RewardsContractFactory };
//# sourceMappingURL=RewardsContractFactory.js.map